/*
    (C) 1995-97 AROS - The Amiga Research OS
    $Id: makedosentry.c,v 1.1.1.1 2005/03/15 15:54:40 laire Exp $

    Desc: Creates an entry for the dos list.
    Lang: english
*/
#include <exec/memory.h>
#include <proto/exec.h>
#include "dos_intern.h"

/*****************************************************************************

    NAME */
#include <proto/dos.h>

	AROS_LH2(struct DosList *, MakeDosEntry,

/*  SYNOPSIS */
	AROS_LHA(CONST_STRPTR, name, D1),
	AROS_LHA(LONG,         type, D2),

/*  LOCATION */
	struct DosLibrary *, DOSBase, 116, Dos)

/*  FUNCTION
	Create an entry for the dos list. Depending on the type this may
	be a device a volume or an assign node.

    INPUTS
	name - pointer to name
	type - type of list entry to create

    RESULT

    NOTES

    EXAMPLE

    BUGS

    SEE ALSO

    INTERNALS

    HISTORY
	29-10-95    digulla automatically created from
			    dos_lib.fd and clib/dos_protos.h

*****************************************************************************/
{
    AROS_LIBFUNC_INIT
    AROS_LIBBASE_EXT_DECL(struct DosLibrary *,DOSBase)

    STRPTR s3;
    ULONG len = 0;
    struct DosList *dl;

    DEBUG_DOSLIST(dprintf("MakeDosEntry: name = \"%s\", type = %ld\n", name, type));

#if !USE_EARLYDOS
    FAILFORTASKNP(return NULL;)
#endif

    dl=(struct DosList *)dosAllocMem(sizeof(struct DosList),MEMF_PUBLIC|MEMF_CLEAR);
    if (dl!=NULL)
    {
	while (len<255 && name[len]) len++;

	/* 02-Aug-2003 bugfix: Must use AllocVec here!
	 * Filesystem's might swap this ptr! - Piru
	 */
	s3=(STRPTR)AllocVec(len+2,MEMF_PUBLIC);
	if (s3!=NULL)
	{
	    /* Compatibility */
	    dl->dol_OldName=MKBADDR(s3);
	    *s3++=len;

	    CopyMem((STRPTR) name, s3, len);
	    s3[len] = 0;	/* Because the docs say so. */

#ifndef AROS_DOS_COMPATIBLE

	    dl->dol_DevName=s3;

#endif /* !AROS_DOS_COMPATIBLE */

	    dl->dol_Type=type;

	    DEBUG_DOSLIST(dprintf("MakeDosEntry: res = $%lx\n", (ULONG) dl));
	
	    return dl;
	} else
	    SetIoErr(ERROR_NO_FREE_STORE);

	dosFreeMem(dl,sizeof(struct DosList));

    } else
        SetIoErr(ERROR_NO_FREE_STORE);

    DEBUG_DOSLIST(dprintf("MakeDosEntry: failed\n"));

    return NULL;
    AROS_LIBFUNC_EXIT
} /* MakeDosEntry */
